﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace PEDataWin.Controls
{

    // Description:		An ImageButton control with various painting styles
    // Date Created:	November 2010
    // Author:			Kevin Carbis
    // Author Email:	kacarbis@gmail.com
    // Websites:		www.staffinder.net/carbisoft

    [DefaultEvent("Click")]
    public class ImageButton : UserControl
    {
        private ImageButtonStyle _ButtonStyle;
        private ImageButtonStyle _HoverStyle;
        private ImageButtonStyle _PressedStyle;
        private ImageButtonStyle _DisabledStyle;

        private ImageButtonStyle _CheckedStyle;
        private Image _Image;

        private string _Text;
        private bool _Checked;
        private bool _Hover;
        private bool _Pressed;
        private bool _ShowShadow = true;
        private int _CornerRadius = 3;

        private ImageButtonCorners _Corners = ImageButtonCorners.All;
        private ToolTip _tooltip = new ToolTip();
        private TextImageRelation _TextImageRelation = TextImageRelation.ImageAboveText;
        private ContentAlignment _TextAlignment = ContentAlignment.TopCenter;

        public ImageButton()
        {
            MouseUp += ImageButton_MouseUp;
            MouseLeave += ImageButton_MouseLeave;
            MouseEnter += ImageButton_MouseEnter;
            MouseDown += ImageButton_MouseDown;
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.ResizeRedraw, true);
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            this.Size = new Size(42, 42);
            this.BackColor = Color.Transparent;

            _ButtonStyle = new ImageButtonStyle(this, FromHex("#C1D5F1"), FromHex("#CFE0F7"), FromHex("#E3EDFB"), FromHex("#B9D0ED"), FromHex("#C8D9ED"), FromHex("#E7F2FF"), this.Font, this.ForeColor);
            _HoverStyle = new ImageButtonStyle(this, FromHex("#FFD646"), FromHex("#FFEAAC"), FromHex("#FFF2C7"), FromHex("#C2A978"), FromHex("#FFFDDB"), FromHex("#FFE793"), this.Font, this.ForeColor);
            _PressedStyle = new ImageButtonStyle(this, FromHex("#F88F2C"), FromHex("#FDF1B0"), FromHex("#F9C65A"), FromHex("#8E8165"), FromHex("#FDD5A8"), FromHex("#FBB062"), this.Font, this.ForeColor);
            _DisabledStyle = new ImageButtonStyle(this, FromHex("#E0E4E8"), FromHex("#E8EBEF"), FromHex("#F1F3F5"), FromHex("#C5D1DE"), FromHex("#F0F3F6"), FromHex("#EAEDF1"), this.Font, this.ForeColor);
            _CheckedStyle = new ImageButtonStyle(this, FromHex("#F9AA45"), FromHex("#FDEA9D"), FromHex("#F9C65A"), FromHex("#8E8165"), FromHex("#F8DBB7"), FromHex("#FED18E"), this.Font, this.ForeColor);
        }
        protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
        {
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            e.Graphics.FillRectangle(new SolidBrush(this.BackColor), ClientRectangle);

            //Shrink the Client Rectangle based on the padding values instead of adding/subtracting them everywhere.  This is the area to paint the contents
            Rectangle ContentRec = Rectangle.FromLTRB(ClientRectangle.Left + Padding.Left, ClientRectangle.Top + Padding.Top, ClientRectangle.Right - Padding.Right, ClientRectangle.Bottom - Padding.Bottom);

            ImageButtonStyle btnStyle = null;
            if (!this.Enabled)
            {
                btnStyle = _DisabledStyle;
            }
            else if (_Pressed)
            {
                btnStyle = _PressedStyle;
            }
            else if (_Hover)
            {
                if (_Checked)
                {
                    btnStyle = _PressedStyle;
                }
                else
                {
                    btnStyle = _HoverStyle;
                }
            }
            else if (_Checked)
            {
                btnStyle = _CheckedStyle;
            }
            else
            {
                btnStyle = _ButtonStyle;
            }
            //draw the gradient background and borders
            DrawButton(e.Graphics, ClientRectangle, btnStyle);

            //determine the location of the image and text
            Rectangle ImageRec = Rectangle.Empty;
            Rectangle TextRec = Rectangle.Empty;
            if (_Image != null)
            {
                switch (_TextImageRelation)
                {
                    case System.Windows.Forms.TextImageRelation.Overlay:
                        {
                            ImageRec = ContentRec;
                            TextRec = ContentRec;
                            break;
                        }
                    case System.Windows.Forms.TextImageRelation.ImageBeforeText:
                        {
                            ImageRec = Rectangle.FromLTRB(ContentRec.Left, ContentRec.Top, ContentRec.Left + _Image.Size.Width, ContentRec.Bottom);
                            TextRec = Rectangle.FromLTRB(ImageRec.Right + 1, ImageRec.Top, ContentRec.Right, ContentRec.Bottom);
                            break;
                        }
                    case System.Windows.Forms.TextImageRelation.TextAboveImage:
                        {
                            ImageRec = Rectangle.FromLTRB(ContentRec.Left, ContentRec.Bottom - _Image.Size.Height, ContentRec.Right, ContentRec.Bottom);
                            TextRec = Rectangle.FromLTRB(ContentRec.Left, ContentRec.Top, ContentRec.Right, ImageRec.Top - 1);
                            break;
                        }
                    case System.Windows.Forms.TextImageRelation.TextBeforeImage:
                        {
                            ImageRec = Rectangle.FromLTRB(ContentRec.Right - _Image.Size.Width, ContentRec.Top, ContentRec.Right, ContentRec.Bottom);
                            TextRec = Rectangle.FromLTRB(ContentRec.Left, ContentRec.Top, ImageRec.Left - 1, ContentRec.Bottom);
                            break;
                        }
                    case System.Windows.Forms.TextImageRelation.ImageAboveText:
                        {
                            ImageRec = Rectangle.FromLTRB(ContentRec.Left, ContentRec.Top, ContentRec.Right, ContentRec.Top + _Image.Height);
                            TextRec = Rectangle.FromLTRB(ContentRec.Left, ImageRec.Bottom + 1, ContentRec.Right, ContentRec.Bottom);
                            break;
                        }
                }
            }
            else
            {
                //no image so just make the whole button the text area
                ImageRec = Rectangle.Empty;
                TextRec = ContentRec;
            }

            SizeF tSize = new SizeF(0, 0);
            if (!string.IsNullOrEmpty(_Text))
            {
                //first measure the text dimensions using the control width so the height is adjusted for multi line.
                tSize = e.Graphics.MeasureString(_Text, btnStyle.Font, TextRec.Width);
            }

            StringFormat txtFormat = GetStringFormat(TextAlignment);

            if (Image != null)
            {
                System.Drawing.Point pt = default(System.Drawing.Point);
                pt.X = Convert.ToInt32(ImageRec.Left + ((ImageRec.Width - Image.Width) / 2));
                pt.Y = Convert.ToInt32(ImageRec.Top + ((ImageRec.Height - Image.Height) / 2));
                if (!this.Enabled)
                {
                    ControlPaint.DrawImageDisabled(e.Graphics, Image, pt.X, pt.Y, Color.Transparent);
                }
                else
                {
                    e.Graphics.DrawImage(Image, pt);
                }
                if (!string.IsNullOrEmpty(_Text))
                {
                    e.Graphics.DrawString(_Text, btnStyle.Font, new SolidBrush(btnStyle.ForeColor), TextRec, txtFormat);
                }
            }
            else if (!string.IsNullOrEmpty(_Text))
            {
                System.Drawing.Point pt = default(System.Drawing.Point);
                pt.X = Convert.ToInt32((TextRec.Width - tSize.Width) / 2);
                pt.Y = Convert.ToInt32((TextRec.Height - tSize.Height) / 2);
                e.Graphics.DrawString(_Text, btnStyle.Font, new SolidBrush(btnStyle.ForeColor), TextRec, txtFormat);
            }

        }
        private StringFormat GetStringFormat(ContentAlignment align)
        {
            StringFormat sf = new StringFormat();
            switch (align)
            {
                case ContentAlignment.TopLeft:
                    sf.LineAlignment = StringAlignment.Near;
                    sf.Alignment = StringAlignment.Near;
                    break;
                case (ContentAlignment.TopCenter):
                    {
                        sf.LineAlignment = StringAlignment.Near;
                        sf.Alignment = StringAlignment.Center;
                        break;
                    }
                case (ContentAlignment.TopRight):
                    {
                        sf.LineAlignment = StringAlignment.Near;
                        sf.Alignment = StringAlignment.Far;
                        break;
                    }
                case (ContentAlignment.MiddleLeft):
                    {
                        sf.LineAlignment = StringAlignment.Center;
                        sf.Alignment = StringAlignment.Near;
                        break;
                    }
                case (ContentAlignment.MiddleCenter):
                    {
                        sf.LineAlignment = StringAlignment.Center;
                        sf.Alignment = StringAlignment.Center;
                        break;
                    }
                case (ContentAlignment.MiddleRight):
                    {
                        sf.LineAlignment = StringAlignment.Center;
                        sf.Alignment = StringAlignment.Far;
                        break;
                    }
                case (ContentAlignment.BottomLeft):
                    {
                        sf.LineAlignment = StringAlignment.Far;
                        sf.Alignment = StringAlignment.Near;
                        break;
                    }
                case (ContentAlignment.BottomCenter):
                    {
                        sf.LineAlignment = StringAlignment.Far;
                        sf.Alignment = StringAlignment.Center;
                        break;
                    }
                case (ContentAlignment.BottomRight):
                    {
                        sf.LineAlignment = StringAlignment.Far;
                        sf.Alignment = StringAlignment.Far;
                        break;
                    }
            }
            return sf;
        }

        private void DrawPressedShadow(Graphics g, Rectangle r)
        {
            Rectangle shadow = Rectangle.FromLTRB(r.Left, r.Top, r.Right, r.Top + 4);

            if (shadow.Width <= 0 | shadow.Height <= 0)
                return;

            using (GraphicsPath path = RoundRectangle(shadow, _CornerRadius, ImageButtonCorners.NorthEast | ImageButtonCorners.NorthWest))
            {
                using (LinearGradientBrush b = new LinearGradientBrush(shadow, Color.FromArgb(50, Color.Black), Color.FromArgb(0, Color.Black), 90))
                {
                    b.WrapMode = WrapMode.TileFlipXY;

                    g.FillPath(b, path);
                }
            }
        }
        private void DrawButton(Graphics g, Rectangle bounds, ImageButtonStyle ColorStyle)
        {
            if (bounds.Height <= 0 | bounds.Width <= 0)
            {
                return;
            }

            Rectangle outerR = Rectangle.FromLTRB(bounds.Left, bounds.Top, bounds.Right - 1, bounds.Bottom - 1);
            Rectangle innerR = Rectangle.FromLTRB(bounds.Left + 1, bounds.Top + 1, bounds.Right - 2, bounds.Bottom - 2);
            Rectangle glossyR = Rectangle.FromLTRB(bounds.Left + 1, bounds.Top + 1, bounds.Right - 2, bounds.Top + Convert.ToInt32(bounds.Height * 0.36));

            using (GraphicsPath boundsPath = RoundRectangle(outerR, _CornerRadius, Corners))
            {
                using (SolidBrush brus = new SolidBrush(ColorStyle.BackColorOuter))
                {
                    g.FillPath(brus, boundsPath);
                }

                //#Region "Main Bg"
                using (GraphicsPath path = new GraphicsPath())
                {
                    path.AddEllipse(new Rectangle(bounds.Left, bounds.Top, bounds.Width, bounds.Height * 2));
                    path.CloseFigure();
                    using (PathGradientBrush gradient = new PathGradientBrush(path))
                    {
                        gradient.WrapMode = WrapMode.Clamp;
                        gradient.CenterPoint = new PointF(Convert.ToSingle(bounds.Left + bounds.Width / 2), Convert.ToSingle(bounds.Bottom));
                        gradient.CenterColor = ColorStyle.BackColorCenter;
                        gradient.SurroundColors = new Color[] { ColorStyle.BackColorOuter };

                        Blend blend = new Blend(3);
                        blend.Factors = new float[] {
						0f,
						0.8f,
						0f
					};
                        blend.Positions = new float[] {
						0f,
						0.3f,
						1f
					};

                        Region lastClip = g.Clip;
                        Region newClip = new Region(boundsPath);
                        newClip.Intersect(lastClip);
                        g.SetClip(newClip.GetBounds(g));
                        g.FillPath(gradient, path);
                        g.Clip = lastClip;
                    }
                }
                //#End Region

                //Glossy effect
                using (GraphicsPath path = RoundRectangle(glossyR, _CornerRadius, Corners))
                {
                    if (glossyR.Width > 0 && glossyR.Height > 0)
                    {
                        using (LinearGradientBrush b = new LinearGradientBrush(glossyR, ColorStyle.GlossySouth, ColorStyle.GlossyNorth, 90))
                        {
                            b.WrapMode = WrapMode.TileFlipXY;
                            g.FillPath(b, path);
                        }
                    }
                }

                //Border
                using (Pen p = new Pen(ColorStyle.BorderOuter))
                {
                    g.DrawPath(p, boundsPath);
                }

                //Inner border
                using (GraphicsPath path = RoundRectangle(innerR, _CornerRadius, Corners))
                {
                    using (Pen p = new Pen(ColorStyle.BorderInner))
                    {
                        g.DrawPath(p, path);
                    }
                }
            }

            if ((_Pressed | _Checked) & _ShowShadow)
            {
                DrawPressedShadow(g, innerR);
            }
        }

        private GraphicsPath RoundRectangle(Rectangle r, int radius, ImageButtonCorners corners)
        {
            GraphicsPath path = new GraphicsPath();
            if (r.Width <= 0 | r.Height <= 0)
                return path;

            int d = radius * 2;

            int nw = ((corners & ImageButtonCorners.NorthWest) == ImageButtonCorners.NorthWest ? d : 0);
            int ne = ((corners & ImageButtonCorners.NorthEast) == ImageButtonCorners.NorthEast ? d : 0);
            int se = ((corners & ImageButtonCorners.SouthEast) == ImageButtonCorners.SouthEast ? d : 0);
            int sw = ((corners & ImageButtonCorners.SouthWest) == ImageButtonCorners.SouthWest ? d : 0);

            path.AddLine(r.Left + nw, r.Top, r.Right - ne, r.Top);

            if (ne > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Right - ne, r.Top, r.Right, r.Top + ne), -90, 90);
            }

            path.AddLine(r.Right, r.Top + ne, r.Right, r.Bottom - se);

            if (se > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Right - se, r.Bottom - se, r.Right, r.Bottom), 0, 90);
            }

            path.AddLine(r.Right - se, r.Bottom, r.Left + sw, r.Bottom);

            if (sw > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Left, r.Bottom - sw, r.Left + sw, r.Bottom), 90, 90);
            }

            path.AddLine(r.Left, r.Bottom - sw, r.Left, r.Top + nw);

            if (nw > 0)
            {
                path.AddArc(Rectangle.FromLTRB(r.Left, r.Top, r.Left + nw, r.Top + nw), 180, 90);
            }

            path.CloseFigure();
            return path;
        }
        private Color FromHex(string hex)
        {
            if (hex.StartsWith("#"))
            {
                hex = hex.Substring(1);

                if ((hex.Length != 6))
                    throw new Exception("Color not valid");

                return Color.FromArgb(int.Parse(hex.Substring(0, 2), System.Globalization.NumberStyles.HexNumber), int.Parse(hex.Substring(2, 2), System.Globalization.NumberStyles.HexNumber), int.Parse(hex.Substring(4, 2), System.Globalization.NumberStyles.HexNumber));
            }
            return Color.White;
        }

        private void ImageButton_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            _Pressed = true;
            Invalidate();
        }
        private void ImageButton_MouseEnter(object sender, System.EventArgs e)
        {
            _Hover = true;
            Invalidate();
        }
        private void ImageButton_MouseLeave(object sender, System.EventArgs e)
        {
            _Hover = false;
            Invalidate();
        }
        private void ImageButton_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            _Pressed = false;
            Invalidate();
        }

        #region Properties

        /// <summary>
        /// The style of the button when it is in its normal unselected and unchecked state.
        /// </summary>
        [Description("The style of the button when its in a normal state.")]
        public ImageButtonStyle ButtonStyle
        {
            get { return _ButtonStyle; }
            set
            {
                _ButtonStyle = value;
                Invalidate();
            }
        }
        /// <summary>
        /// The style of the button when it is in a checked state.
        /// </summary>
        [Description("The style of the button when it is in a checked state.")]
        public ImageButtonStyle CheckedStyle
        {
            get { return _CheckedStyle; }
            set
            {
                _CheckedStyle = value;
                Invalidate();
            }
        }
        /// <summary>
        /// The style of the button when the mouse is over the button.
        /// </summary>
        [Description("The style of the button when the mouse is over the button.")]
        public ImageButtonStyle HoverStyle
        {
            get { return _HoverStyle; }
            set { _HoverStyle = value; }
        }
        /// <summary>
        /// The style of the button when the button is pressed.
        /// </summary>
        [Description("The style of the button when the button is pressed.")]
        public ImageButtonStyle PressedStyle
        {
            get { return _PressedStyle; }
            set { _PressedStyle = value; }
        }
        /// <summary>
        /// The style of the button when the button is Disabled.
        /// </summary>
        [Description("The style of the button when the button is pressed.")]
        public ImageButtonStyle DisabledStyle
        {
            get { return _DisabledStyle; }
            set { _DisabledStyle = value; }
        }

        /// <summary>
        /// Gets or Sets if the shadow is drawn when pressed.
        /// </summary>
        [Description("Gets or Sets if the shadow is drawn when pressed."), DefaultValue(true)]
        public bool ShowShadow
        {
            get { return _ShowShadow; }
            set
            {
                _ShowShadow = value;
                Invalidate();
            }
        }
        [Browsable(true), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [DefaultValue(false)]
        public bool Checked
        {
            get { return _Checked; }
            set
            {
                _Checked = value;
                Invalidate();
            }
        }
        /// <summary>Specifies which corners to apply rounding for</summary>
        [Browsable(true), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [DefaultValue(ImageButtonCorners.All)]
        public ImageButtonCorners Corners
        {
            get { return _Corners; }
            set { _Corners = value; }
        }
        /// <summary>Gets or sets the radius of the corners of the control.</summary>
        [Category("Appearance"), Description("This feature will round the corners of the control.")]
        [DefaultValue(3)]
        public int CornerRadius
        {
            get { return _CornerRadius; }
            set { _CornerRadius = value; }
        }
        public Image Image
        {
            get { return _Image; }
            set
            {
                _Image = value;
                Invalidate();
            }
        }

        [Browsable(true), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public override string Text
        {
            get { return _Text; }
            set
            {
                _Text = value;
                Invalidate();
            }
        }

        [Browsable(true), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        public string ToolTipText
        {
            get { return _tooltip.GetToolTip(this); }
            set { _tooltip.SetToolTip(this, value); }
        }

        [Browsable(true), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [DefaultValue(TextImageRelation.ImageAboveText)]
        public TextImageRelation TextImageRelation
        {
            get { return _TextImageRelation; }
            set { _TextImageRelation = value; Invalidate(); }
        }

        [Browsable(true), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [DefaultValue(ContentAlignment.TopCenter)]
        public ContentAlignment TextAlignment
        {
            get { return _TextAlignment; }
            set { _TextAlignment = value; Invalidate(); }
        }
        
        [Browsable(true), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [DefaultValue(true)]
        public new bool Enabled
        {
            get { return base.Enabled; }
            set { base.Enabled = value; Invalidate(); }
        }

        #endregion

    }

    public enum ImageButtonCorners
    {
        None = 0,
        NorthWest = 2,
        NorthEast = 4,
        SouthEast = 8,
        SouthWest = 16,
        All = NorthWest | NorthEast | SouthEast | SouthWest,
        North = NorthWest | NorthEast,
        South = SouthEast | SouthWest,
        East = NorthEast | SouthEast,
        West = NorthWest | SouthWest
    }

    [TypeConverter(typeof(ButtonStyleTypeConverter))]
    public class ImageButtonStyle
    {
        public event PropertyChangedEventHandler PropertyChanged;
        public delegate void PropertyChangedEventHandler(string PropertyName);
        private Control _Owner;
        private Color _BackColorOuter;
        private Color _BackColorCenter;
        private Color _BorderInner;
        private Color _BorderOuter;
        private Color _GlossyNorth;
        private Color _GlossySouth;
        private Font _Font;

        private Color _ForeColor;
        public ImageButtonStyle()
        {
            PropertyChanged += ImageButtonStyle_PropertyChanged;
        }
        public ImageButtonStyle(Control Owner, Color BackColorOuter, Color BackColorCenter, Color BorderInner, Color BorderOuter, Color GlossyNorth, Color GlossySouth, Font Font, Color ForeColor)
        {
            PropertyChanged += ImageButtonStyle_PropertyChanged;
            _Owner = Owner;
            this.BackColorOuter = BackColorOuter;
            this.BackColorCenter = BackColorCenter;
            this.BorderInner = BorderInner;
            this.BorderOuter = BorderOuter;
            this.GlossyNorth = GlossyNorth;
            this.GlossySouth = GlossySouth;
            this.Font = Font;
            this.ForeColor = ForeColor;
        }

        public Color BackColorOuter
        {
            get { return _BackColorOuter; }
            set
            {
                _BackColorOuter = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("BackColorOuter");
                }
            }
        }
        public Color BackColorCenter
        {
            get { return _BackColorCenter; }
            set
            {
                _BackColorCenter = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("BackColorCenter");
                }
            }
        }
        public Color BorderInner
        {
            get { return _BorderInner; }
            set
            {
                _BorderInner = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("BorderInner");
                }
            }
        }
        public Color BorderOuter
        {
            get { return _BorderOuter; }
            set
            {
                _BorderOuter = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("BorderOuter");
                }
            }
        }
        public Color GlossyNorth
        {
            get { return _GlossyNorth; }
            set
            {
                _GlossyNorth = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("GlossyNorth");
                }
            }
        }
        public Color GlossySouth
        {
            get { return _GlossySouth; }
            set
            {
                _GlossySouth = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("GlossySouth");
                }
            }
        }
        public Font Font
        {
            get { return _Font; }
            set
            {
                _Font = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("Font");
                }
            }
        }
        public Color ForeColor
        {
            get { return _ForeColor; }
            set
            {
                _ForeColor = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged("ForeColor");
                }
            }
        }
        private void ImageButtonStyle_PropertyChanged(string PropertyName)
        {
            if (_Owner != null)
            {
                _Owner.Invalidate();
            }
        }
    }
    public class ButtonStyleTypeConverter : TypeConverter
    {
        public ButtonStyleTypeConverter()
        {
        }
        public override bool GetPropertiesSupported(System.ComponentModel.ITypeDescriptorContext context)
        {
            return true;
            //MyBase.GetPropertiesSupported(context)
        }
        public override System.ComponentModel.PropertyDescriptorCollection GetProperties(System.ComponentModel.ITypeDescriptorContext context, object value, System.Attribute[] attributes)
        {
            return TypeDescriptor.GetProperties(typeof(ImageButtonStyle));
        }
    }

}
