﻿' Description:		An ImageButton control with various painting styles
' Date Created:	November 2010
' Author:			Kevin Carbis
' Author Email:	kcarbis@comcast.net / kevin@staffinder.net
' Websites:			www.staffinder.net/carbisoft
' Credits:			This is the same as the RibbonButton from the Office 2007 Ribbon control from www.menendezpoo.com.
'						I needed a button that duplicated the appearance and functionality of the RibbonButton.
' License:			This control if free to use in any application however you cannot distribute or sell this control or bundle it with any other control or software package.
' Version			1.0 - 4/28/2011 Initial Release

Imports System.ComponentModel
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Drawing.Imaging
Imports System.Runtime.InteropServices


<DefaultEvent("Click")> _
Public Class ImageButton
   Inherits UserControl

   Private _ButtonStyle As ImageButtonStyle
   Private _HoverStyle As ImageButtonStyle
   Private _PressedStyle As ImageButtonStyle
   Private _DisabledStyle As ImageButtonStyle
   Private _CheckedStyle As ImageButtonStyle

   Private _Image As Image
   Private _Text As String

   Private _Checked As Boolean
   Private _Hover As Boolean
   Private _Pressed As Boolean
   Private _ShowShadow As Boolean = True
   Private _CornerRadius As Integer = 3
   Private _DisabledImageColorMatrix As ColorMatrix

   Private _Corners As ImageButtonCorners = ImageButtonCorners.All
   Private _tooltip As New ToolTip
   Private _TextImageRelation As TextImageRelation = TextImageRelation.ImageAboveText
   Private _TextAlignment As ContentAlignment = ContentAlignment.TopCenter

   Public Sub New()
      SetStyle(ControlStyles.OptimizedDoubleBuffer, True)
      SetStyle(ControlStyles.AllPaintingInWmPaint, True)
      SetStyle(ControlStyles.ResizeRedraw, True)
      SetStyle(ControlStyles.UserPaint, True)
      SetStyle(ControlStyles.SupportsTransparentBackColor, True)
      Me.Size = New Size(42, 42)
      Me.BackColor = Color.Transparent

      _ButtonStyle = New ImageButtonStyle(Me, FromHex("#C1D5F1"), FromHex("#CFE0F7"), _
            FromHex("#E3EDFB"), FromHex("#B9D0ED"), _
            FromHex("#C8D9ED"), FromHex("#E7F2FF"), _
            Me.Font, Me.ForeColor)

      _HoverStyle = New ImageButtonStyle(Me, FromHex("#FFD646"), FromHex("#FFEAAC"), _
            FromHex("#FFF2C7"), FromHex("#C2A978"), _
            FromHex("#FFFDDB"), FromHex("#FFE793"), _
            Me.Font, Me.ForeColor)

      _PressedStyle = New ImageButtonStyle(Me, FromHex("#F88F2C"), FromHex("#FDF1B0"), _
            FromHex("#F9C65A"), FromHex("#8E8165"), _
            FromHex("#FDD5A8"), FromHex("#FBB062"), _
            Me.Font, Me.ForeColor)

      _DisabledStyle = New ImageButtonStyle(Me, FromHex("#E0E4E8"), FromHex("#E8EBEF"), _
            FromHex("#F1F3F5"), FromHex("#C5D1DE"), _
            FromHex("#F0F3F6"), FromHex("#EAEDF1"), _
            Me.Font, Me.ForeColor)

      _CheckedStyle = New ImageButtonStyle(Me, FromHex("#F9AA45"), FromHex("#FDEA9D"), _
            FromHex("#F9C65A"), FromHex("#8E8165"), _
            FromHex("#F8DBB7"), FromHex("#FED18E"), _
            Me.Font, Me.ForeColor)
   End Sub
   Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
      e.Graphics.SmoothingMode = SmoothingMode.AntiAlias
      e.Graphics.FillRectangle(New SolidBrush(Me.BackColor), ClientRectangle)

      'Shrink the Clip Rectangle based on the padding values instead of adding/subtracting them everywhere
      Dim ClipRectangle As Rectangle = Rectangle.FromLTRB(e.ClipRectangle.Left + Padding.Left, e.ClipRectangle.Top + Padding.Top, e.ClipRectangle.Right - Padding.Right, e.ClipRectangle.Bottom - Padding.Bottom)

      Dim btnStyle As ImageButtonStyle = Nothing
      If Not Me.Enabled Then
         btnStyle = _DisabledStyle
      ElseIf _Pressed Then
         btnStyle = _PressedStyle
      ElseIf _Hover Then
         If _Checked Then
            btnStyle = _PressedStyle
         Else
            btnStyle = _HoverStyle
         End If
      ElseIf _Checked Then
         btnStyle = _CheckedStyle
      Else
         btnStyle = _ButtonStyle
      End If
      'draw the gradient background
      DrawButton(e.Graphics, ClientRectangle, btnStyle)

      'determine the location of the image and text
      Dim ImageRec As Rectangle = Rectangle.Empty
      Dim TextRec As Rectangle = Rectangle.Empty
      If _Image IsNot Nothing Then
         Select Case _TextImageRelation
            Case System.Windows.Forms.TextImageRelation.Overlay
               If True Then
                  ImageRec = ClipRectangle
                  TextRec = ClipRectangle
                  Exit Select
               End If
            Case System.Windows.Forms.TextImageRelation.ImageBeforeText
               If True Then
                  ImageRec = Rectangle.FromLTRB(ClipRectangle.Left, ClipRectangle.Top, ClipRectangle.Left + _Image.Size.Width, ClipRectangle.Bottom)
                  TextRec = Rectangle.FromLTRB(ImageRec.Right + 1, ImageRec.Top, ClipRectangle.Right, ClipRectangle.Bottom)
                  Exit Select
               End If
            Case System.Windows.Forms.TextImageRelation.TextAboveImage
               If True Then
                  ImageRec = Rectangle.FromLTRB(ClipRectangle.Left, ClipRectangle.Bottom - _Image.Size.Height, ClipRectangle.Right, ClipRectangle.Bottom)
                  TextRec = Rectangle.FromLTRB(ClipRectangle.Left, ClipRectangle.Top, ClipRectangle.Right, ImageRec.Top - 1)
                  Exit Select
               End If
            Case System.Windows.Forms.TextImageRelation.TextBeforeImage
               If True Then
                  ImageRec = Rectangle.FromLTRB(ClipRectangle.Right - _Image.Size.Width, ClipRectangle.Top, ClipRectangle.Right, ClipRectangle.Bottom)
                  TextRec = Rectangle.FromLTRB(ClipRectangle.Left, ClipRectangle.Top, ImageRec.Left - 1, ClipRectangle.Bottom)
                  Exit Select
               End If
            Case System.Windows.Forms.TextImageRelation.ImageAboveText
               If True Then
                  ImageRec = Rectangle.FromLTRB(ClipRectangle.Left, ClipRectangle.Top, ClipRectangle.Right, ClipRectangle.Top + _Image.Height)
                  TextRec = Rectangle.FromLTRB(ClipRectangle.Left, ImageRec.Bottom + 1, ClipRectangle.Right, ClipRectangle.Bottom)
                  Exit Select
               End If
         End Select
      Else
         'no image so just make the whole button the text area
         ImageRec = Rectangle.Empty
         TextRec = ClipRectangle
      End If

      Dim tSize As New SizeF(0, 0)
      If Not String.IsNullOrEmpty(_Text) Then

         'first measure the text dimensions using the control width so the height is adjusted for multi line.
         tSize = e.Graphics.MeasureString(_Text, btnStyle.Font, TextRec.Width)
      End If

      Dim txtFormat As StringFormat = GetStringFormat(_TextAlignment)

      If Image IsNot Nothing Then
         Dim pt As System.Drawing.Point = Nothing
         pt.X = Convert.ToInt32(ImageRec.Left + ((ImageRec.Width - Image.Width) \ 2))
         pt.Y = Convert.ToInt32(ImageRec.Top + ((ImageRec.Height - Image.Height) \ 2))
         If Not Me.Enabled Then
            ControlPaint.DrawImageDisabled(e.Graphics, Image, pt.X, pt.Y, Color.Transparent)
         Else
            e.Graphics.DrawImage(Image, pt)
         End If
         If Not String.IsNullOrEmpty(_Text) Then
            e.Graphics.DrawString(_Text, btnStyle.Font, New SolidBrush(btnStyle.ForeColor), TextRec, txtFormat)
         End If
      ElseIf Not String.IsNullOrEmpty(_Text) Then
         Dim pt As System.Drawing.Point = Nothing
         pt.X = Convert.ToInt32((TextRec.Width - tSize.Width) / 2)
         pt.Y = Convert.ToInt32((TextRec.Height - tSize.Height) / 2)
         e.Graphics.DrawString(_Text, btnStyle.Font, New SolidBrush(btnStyle.ForeColor), TextRec, txtFormat)
      End If
   End Sub

   ''' <summary>
   ''' The style of the button when it is in its normal unselected and unchecked state.
   ''' </summary>
	<Description("The style of the button when its in a normal state.")> _
	Public Property ButtonStyle() As ImageButtonStyle
		Get
			Return _ButtonStyle
		End Get
		Set(ByVal value As ImageButtonStyle)
			_ButtonStyle = value
			Invalidate()
		End Set
	End Property
   ''' <summary>
   ''' The style of the button when it is in a checked state.
   ''' </summary>
	<Description("The style of the button when it is in a checked state.")> _
	Public Property CheckedStyle() As ImageButtonStyle
		Get
			Return _CheckedStyle
		End Get
		Set(ByVal value As ImageButtonStyle)
			_CheckedStyle = value
			Invalidate()
		End Set
	End Property
   ''' <summary>
   ''' The style of the button when the mouse is over the button.
   ''' </summary>
	<Description("The style of the button when the mouse is over the button.")> _
	Public Property HoverStyle() As ImageButtonStyle
		Get
			Return _HoverStyle
		End Get
		Set(ByVal value As ImageButtonStyle)
			_HoverStyle = value
		End Set
	End Property
   ''' <summary>
   ''' The style of the button when the button is pressed.
   ''' </summary>
	<Description("The style of the button when the button is pressed.")> _
	Public Property PressedStyle() As ImageButtonStyle
		Get
			Return _PressedStyle
		End Get
		Set(ByVal value As ImageButtonStyle)
			_PressedStyle = value
		End Set
	End Property

   ''' <summary>
   ''' Gets or Sets if the shadow is drawn when pressed.
   ''' </summary>
	<Description("Gets or Sets if the shadow is drawn when pressed."), DefaultValue(True)> _
	Public Property ShowShadow() As Boolean
		Get
			Return _ShowShadow
		End Get
		Set(ByVal value As Boolean)
			_ShowShadow = value
			Invalidate()
		End Set
	End Property
   Public Property Checked() As Boolean
      Get
         Return _Checked
      End Get
      Set(ByVal value As Boolean)
         _Checked = value
         Invalidate()
      End Set
   End Property
	''' <summary>Specifies which corners to apply rounding for</summary>
	Public Property Corners() As ImageButtonCorners
		Get
			Return _Corners
		End Get
		Set(ByVal value As ImageButtonCorners)
			_Corners = value
		End Set
	End Property
	''' <summary>Gets or sets the radius of the corners of the control.</summary>
	<Category("Appearance"), Description("This feature will round the corners of the control.")> _
	<DefaultValue(3)> _
	Public Property CornerRadius() As Integer
		Get
			Return _CornerRadius
		End Get
		Set(ByVal value As Integer)
			_CornerRadius = value
		End Set
	End Property
   Public Property Image() As Image
      Get
         Return _Image
      End Get
      Set(ByVal value As Image)
         _Image = value
         Invalidate()
      End Set
   End Property

   Private Sub DrawPressedShadow(ByVal g As Graphics, ByVal r As Rectangle)
      Dim shadow As Rectangle = Rectangle.FromLTRB(r.Left, r.Top, r.Right, r.Top + 4)

      If shadow.Width <= 0 Or shadow.Height <= 0 Then Return

      Using path As GraphicsPath = RoundRectangle(shadow, _CornerRadius, ImageButtonCorners.NorthEast Or ImageButtonCorners.NorthWest)
         Using b As New LinearGradientBrush(shadow, Color.FromArgb(50, Color.Black), Color.FromArgb(0, Color.Black), 90)
            b.WrapMode = WrapMode.TileFlipXY

            g.FillPath(b, path)
         End Using
      End Using
   End Sub
   Private Sub DrawButton(ByVal g As Graphics, ByVal bounds As Rectangle, ByVal ColorStyle As ImageButtonStyle)
      If bounds.Height <= 0 Or bounds.Width <= 0 Then
         Exit Sub
      End If

      Dim outerR As Rectangle = Rectangle.FromLTRB(bounds.Left, bounds.Top, bounds.Right - 1, bounds.Bottom - 1)
      Dim innerR As Rectangle = Rectangle.FromLTRB(bounds.Left + 1, bounds.Top + 1, bounds.Right - 2, bounds.Bottom - 2)
      Dim glossyR As Rectangle = Rectangle.FromLTRB(bounds.Left + 1, bounds.Top + 1, bounds.Right - 2, bounds.Top + CInt(bounds.Height * 0.36))

      Using boundsPath As GraphicsPath = RoundRectangle(outerR, _CornerRadius, Corners)
         Using brus As New SolidBrush(ColorStyle.BackColorOuter)
            g.FillPath(brus, boundsPath)
         End Using

         '#Region "Main Bg"
         Using path As New GraphicsPath()
            path.AddEllipse(New Rectangle(bounds.Left, bounds.Top, bounds.Width, bounds.Height * 2))
            path.CloseFigure()
            Using gradient As New PathGradientBrush(path)
               gradient.WrapMode = WrapMode.Clamp
               gradient.CenterPoint = New PointF(Convert.ToSingle(bounds.Left + bounds.Width / 2), Convert.ToSingle(bounds.Bottom))
               gradient.CenterColor = ColorStyle.BackColorCenter
               gradient.SurroundColors = New Color() {ColorStyle.BackColorOuter}

               Dim blend As New Blend(3)
               blend.Factors = New Single() {0.0F, 0.8F, 0.0F}
               blend.Positions = New Single() {0.0F, 0.3F, 1.0F}

               Dim lastClip As Region = g.Clip
               Dim newClip As New Region(boundsPath)
               newClip.Intersect(lastClip)
               g.SetClip(newClip.GetBounds(g))
               g.FillPath(gradient, path)
               g.Clip = lastClip
            End Using
         End Using
         '#End Region

         'Glossy effect
         Using path As GraphicsPath = RoundRectangle(glossyR, _CornerRadius, Corners)
            If glossyR.Width > 0 AndAlso glossyR.Height > 0 Then
               Using b As New LinearGradientBrush(glossyR, ColorStyle.GlossySouth, ColorStyle.GlossyNorth, 90)
                  b.WrapMode = WrapMode.TileFlipXY
                  g.FillPath(b, path)
               End Using
            End If
         End Using

         'Border
         Using p As New Pen(ColorStyle.BorderOuter)
            g.DrawPath(p, boundsPath)
         End Using

         'Inner border
         Using path As GraphicsPath = RoundRectangle(innerR, _CornerRadius, Corners)
            Using p As New Pen(ColorStyle.BorderInner)
               g.DrawPath(p, path)
            End Using
         End Using
      End Using

      If (_Pressed Or _Checked) And _ShowShadow Then
         DrawPressedShadow(g, innerR)
      End If
   End Sub

   Private Function RoundRectangle(ByVal r As Rectangle, ByVal radius As Integer, ByVal corners As ImageButtonCorners) As GraphicsPath
      Dim path As GraphicsPath = New GraphicsPath()
      If r.Width <= 0 Or r.Height <= 0 Then Return path

      Dim d As Integer = radius * 2

      Dim nw As Integer = IIf((corners And ImageButtonCorners.NorthWest) = ImageButtonCorners.NorthWest, d, 0)
      Dim ne As Integer = IIf((corners And ImageButtonCorners.NorthEast) = ImageButtonCorners.NorthEast, d, 0)
      Dim se As Integer = IIf((corners And ImageButtonCorners.SouthEast) = ImageButtonCorners.SouthEast, d, 0)
      Dim sw As Integer = IIf((corners And ImageButtonCorners.SouthWest) = ImageButtonCorners.SouthWest, d, 0)

      path.AddLine(r.Left + nw, r.Top, r.Right - ne, r.Top)

      If ne > 0 Then
         path.AddArc(Rectangle.FromLTRB(r.Right - ne, r.Top, r.Right, r.Top + ne), -90, 90)
      End If

      path.AddLine(r.Right, r.Top + ne, r.Right, r.Bottom - se)

      If se > 0 Then
         path.AddArc(Rectangle.FromLTRB(r.Right - se, r.Bottom - se, r.Right, r.Bottom), 0, 90)
      End If

      path.AddLine(r.Right - se, r.Bottom, r.Left + sw, r.Bottom)

      If sw > 0 Then
         path.AddArc(Rectangle.FromLTRB(r.Left, r.Bottom - sw, r.Left + sw, r.Bottom), 90, 90)
      End If

      path.AddLine(r.Left, r.Bottom - sw, r.Left, r.Top + nw)

      If nw > 0 Then
         path.AddArc(Rectangle.FromLTRB(r.Left, r.Top, r.Left + nw, r.Top + nw), 180, 90)
      End If

      path.CloseFigure()
      Return path
   End Function
   Private Function FromHex(ByVal hex As String) As Color
      If hex.StartsWith("#") Then
         hex = hex.Substring(1)

         If (hex.Length <> 6) Then Throw New Exception("Color not valid")

         Return Color.FromArgb( _
             Integer.Parse(hex.Substring(0, 2), System.Globalization.NumberStyles.HexNumber), _
             Integer.Parse(hex.Substring(2, 2), System.Globalization.NumberStyles.HexNumber), _
             Integer.Parse(hex.Substring(4, 2), System.Globalization.NumberStyles.HexNumber))
      End If
   End Function

   Private Sub ImageButton_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
      _Pressed = True
      Invalidate()
   End Sub
   Private Sub ImageButton_MouseEnter(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.MouseEnter
      _Hover = True
      Invalidate()
   End Sub
   Private Sub ImageButton_MouseLeave(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.MouseLeave
      _Hover = False
      Invalidate()
   End Sub
   Private Sub ImageButton_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
      _Pressed = False
      Invalidate()
   End Sub
   Private Function GetStringFormat(ByVal align As ContentAlignment) As StringFormat
      Dim sf As New StringFormat()
      Select Case align
         Case ContentAlignment.TopLeft
            sf.LineAlignment = StringAlignment.Near
            sf.Alignment = StringAlignment.Near
            Exit Select
         Case (ContentAlignment.TopCenter)
            If True Then
               sf.LineAlignment = StringAlignment.Near
               sf.Alignment = StringAlignment.Center
               Exit Select
            End If
         Case (ContentAlignment.TopRight)
            If True Then
               sf.LineAlignment = StringAlignment.Near
               sf.Alignment = StringAlignment.Far
               Exit Select
            End If
         Case (ContentAlignment.MiddleLeft)
            If True Then
               sf.LineAlignment = StringAlignment.Center
               sf.Alignment = StringAlignment.Near
               Exit Select
            End If
         Case (ContentAlignment.MiddleCenter)
            If True Then
               sf.LineAlignment = StringAlignment.Center
               sf.Alignment = StringAlignment.Center
               Exit Select
            End If
         Case (ContentAlignment.MiddleRight)
            If True Then
               sf.LineAlignment = StringAlignment.Center
               sf.Alignment = StringAlignment.Far
               Exit Select
            End If
         Case (ContentAlignment.BottomLeft)
            If True Then
               sf.LineAlignment = StringAlignment.Far
               sf.Alignment = StringAlignment.Near
               Exit Select
            End If
         Case (ContentAlignment.BottomCenter)
            If True Then
               sf.LineAlignment = StringAlignment.Far
               sf.Alignment = StringAlignment.Center
               Exit Select
            End If
         Case (ContentAlignment.BottomRight)
            If True Then
               sf.LineAlignment = StringAlignment.Far
               sf.Alignment = StringAlignment.Far
               Exit Select
            End If
      End Select
      Return sf
   End Function

   <Browsable(True), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)> _
   Public Overrides Property Text() As String
      Get
         Return _Text
      End Get
      Set(ByVal value As String)
         _Text = value
         Invalidate()
      End Set
   End Property

   <Browsable(True), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)> _
   Public Property ToolTipText() As String
      Get
         Return _tooltip.GetToolTip(Me)
      End Get
      Set(ByVal value As String)
         _tooltip.SetToolTip(Me, value)
      End Set
   End Property

   <Browsable(True), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)> _
  <DefaultValue(TextImageRelation.ImageAboveText)> _
  Public Property TextImageRelation() As TextImageRelation
      Get
         Return _TextImageRelation
      End Get
      Set(ByVal value As TextImageRelation)
         _TextImageRelation = value
         Invalidate()
      End Set
   End Property
   <Browsable(True), EditorBrowsable(EditorBrowsableState.Always), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)> _
   <DefaultValue(ContentAlignment.TopCenter)> _
   Public Property TextAlignment() As ContentAlignment
      Get
         Return _TextAlignment
      End Get
      Set(ByVal value As ContentAlignment)
         _TextAlignment = value
         Invalidate()
      End Set
   End Property

End Class

Public Enum ImageButtonCorners
   None = 0
   NorthWest = 2
   NorthEast = 4
   SouthEast = 8
   SouthWest = 16
   All = NorthWest Or NorthEast Or SouthEast Or SouthWest
   North = NorthWest Or NorthEast
   South = SouthEast Or SouthWest
   East = NorthEast Or SouthEast
   West = NorthWest Or SouthWest
End Enum

<TypeConverter(GetType(ButtonStyleTypeConverter))> _
Public Class ImageButtonStyle
   Public Event PropertyChanged As PropertyChangedEventHandler
   Public Delegate Sub PropertyChangedEventHandler(ByVal PropertyName As String)
   Private _Owner As Control
   Private _BackColorOuter As Color
   Private _BackColorCenter As Color
   Private _BorderInner As Color
   Private _BorderOuter As Color
   Private _GlossyNorth As Color
   Private _GlossySouth As Color
   Private _Font As Font
   Private _ForeColor As Color

   Public Sub New()
   End Sub
   Public Sub New(ByVal Owner As Control, ByVal BackColorOuter As Color, ByVal BackColorCenter As Color, _
            ByVal BorderInner As Color, ByVal BorderOuter As Color, _
            ByVal GlossyNorth As Color, ByVal GlossySouth As Color, _
            ByVal Font As Font, ByVal ForeColor As Color)
      _Owner = Owner
      Me.BackColorOuter = BackColorOuter
      Me.BackColorCenter = BackColorCenter
      Me.BorderInner = BorderInner
      Me.BorderOuter = BorderOuter
      Me.GlossyNorth = GlossyNorth
      Me.GlossySouth = GlossySouth
      Me.Font = Font
      Me.ForeColor = ForeColor
   End Sub

   Public Property BackColorOuter() As Color
      Get
         Return _BackColorOuter
      End Get
      Set(ByVal value As Color)
         _BackColorOuter = value
         RaiseEvent PropertyChanged("BackColorOuter")
      End Set
   End Property
   Public Property BackColorCenter() As Color
      Get
         Return _BackColorCenter
      End Get
      Set(ByVal value As Color)
         _BackColorCenter = value
         RaiseEvent PropertyChanged("BackColorCenter")
      End Set
   End Property
   Public Property BorderInner() As Color
      Get
         Return _BorderInner
      End Get
      Set(ByVal value As Color)
         _BorderInner = value
         RaiseEvent PropertyChanged("BorderInner")
      End Set
   End Property
   Public Property BorderOuter() As Color
      Get
         Return _BorderOuter
      End Get
      Set(ByVal value As Color)
         _BorderOuter = value
         RaiseEvent PropertyChanged("BorderOuter")
      End Set
   End Property
   Public Property GlossyNorth() As Color
      Get
         Return _GlossyNorth
      End Get
      Set(ByVal value As Color)
         _GlossyNorth = value
         RaiseEvent PropertyChanged("GlossyNorth")
      End Set
   End Property
   Public Property GlossySouth() As Color
      Get
         Return _GlossySouth
      End Get
      Set(ByVal value As Color)
         _GlossySouth = value
         RaiseEvent PropertyChanged("GlossySouth")
      End Set
   End Property
   Public Property Font() As Font
      Get
         Return _Font
      End Get
      Set(ByVal value As Font)
         _Font = value
         RaiseEvent PropertyChanged("Font")
      End Set
   End Property
   Public Property ForeColor() As Color
      Get
         Return _ForeColor
      End Get
      Set(ByVal value As Color)
         _ForeColor = value
         RaiseEvent PropertyChanged("ForeColor")
      End Set
   End Property
   Private Sub ImageButtonStyle_PropertyChanged(ByVal PropertyName As String) Handles Me.PropertyChanged
      If _Owner IsNot Nothing Then
         _Owner.Invalidate()
      End If
   End Sub
End Class
Public Class ButtonStyleTypeConverter
   Inherits TypeConverter
   Public Sub New()
   End Sub
   Public Overrides Function GetPropertiesSupported(ByVal context As System.ComponentModel.ITypeDescriptorContext) As Boolean
      Return True 'MyBase.GetPropertiesSupported(context)
   End Function
   Public Overrides Function GetProperties(ByVal context As System.ComponentModel.ITypeDescriptorContext, ByVal value As Object, ByVal attributes() As System.Attribute) As System.ComponentModel.PropertyDescriptorCollection
      Return TypeDescriptor.GetProperties(GetType(ImageButtonStyle))
   End Function
End Class